package cn.ezhizao.project.business.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.BizStorageRetrieval;
import cn.ezhizao.project.business.service.IBizStorageRetrievalService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 零存零取Controller
 *
 * @author ezhizao
 * @date 2024-10-11
 */
@RestController
@RequestMapping("/business/storageRetrieval")
public class BizStorageRetrievalController extends BaseController
{
    @Resource
    private IBizStorageRetrievalService bizStorageRetrievalService;

    /**
     * 查询零存零取列表
     */
    @GetMapping("/list")
    public TableDataInfo list(BizStorageRetrieval bizStorageRetrieval) throws NoSuchFieldException, IllegalAccessException
    {
        startPage();
        if(bizStorageRetrieval.getStartStorageTime()!=null) {
            bizStorageRetrieval.setStartStorageTime(bizStorageRetrieval.getStartStorageTime()+ " 00:00:00");
        }
        if(bizStorageRetrieval.getEndStorageTime()!=null) {
            bizStorageRetrieval.setEndStorageTime(bizStorageRetrieval.getEndStorageTime()+ " 23:59:59");
        }
        if(bizStorageRetrieval.getStartRetrievalTime()!=null) {
            bizStorageRetrieval.setStartRetrievalTime(bizStorageRetrieval.getStartRetrievalTime()+ " 00:00:00");
        }
        if(bizStorageRetrieval.getEndRetrievalTime()!=null) {
            bizStorageRetrieval.setEndRetrievalTime(bizStorageRetrieval.getEndRetrievalTime()+ " 23:59:59");
        }
        List<BizStorageRetrieval> list = bizStorageRetrievalService.getList(bizStorageRetrieval);
        return getDataTable(list);
    }

    /**
     * 导出零存零取列表
     */
    @PreAuthorize("@ss.hasPermi('business:storageRetrieval:export')")
    @Log(title = "零存零取", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizStorageRetrieval bizStorageRetrieval) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizStorageRetrieval);
        List<BizStorageRetrieval> list = bizStorageRetrievalService.getList(bizStorageRetrieval);
        ExcelUtil<BizStorageRetrieval> util = new ExcelUtil<BizStorageRetrieval>(BizStorageRetrieval.class);
        util.exportExcel(response, list, "零存零取数据");
    }

    /**
     * 获取零存零取详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:storageRetrieval:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizStorageRetrievalService.getById(id));
    }

    /**
     * 新增零存零取
     */
    @Log(title = "零存零取", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizStorageRetrieval bizStorageRetrieval) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizStorageRetrieval);
        return toAjax(bizStorageRetrievalService.save(bizStorageRetrieval));
    }

    /**
     * 修改零存零取
     */
    @Log(title = "零存零取", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizStorageRetrieval bizStorageRetrieval) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizStorageRetrieval);
        return toAjax(bizStorageRetrievalService.updateById(bizStorageRetrieval));
    }

    /**
     * 删除零存零取
     */
    @PreAuthorize("@ss.hasPermi('business:storageRetrieval:remove')")
    @Log(title = "零存零取", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizStorageRetrievalService.removeBatchByIds(ids));
    }
}
